# -*- coding: utf-8 -*-
import os


class BlingConfig(object):
    @property
    def config_path(self):
        return os.path.abspath('./.bling.json')

    @property
    def required_keys(self):
        return [
            'run',
            'start',
            'name',
            'packer',
        ]

    @property
    def optional_keys(self):
        return {
            'env': {},
        }


class CommandConfig(object):
    @property
    def run_command(self):
        """Command to execute when `docker run` is invoked."""
        return '/bin/bash'


class LockConfig(object):
    @property
    def lock_file_path(self):
        """Local path to store the md5 hash after checking for Packer changes."""
        return os.path.abspath('./.bling.lock')

    @property
    def tar_file_path(self):
        """Temporary tar file store location used to ref before the md5sum."""
        return os.path.abspath('./.lock.tar')


class PackerConfig(object):
    @property
    def tmp_root(self):
        return os.path.abspath('./.bling')

    @property
    def tmp_path(self):
        return os.path.join(self.tmp_root, 'template.updated.json')

    @property
    def config_file(self):
        return 'template.json'


class DockerConfig(object):
    @property
    def default_local_tag(self):
        return 'local'

    @property
    def invalid_tag_chars(self):
        return '!"#$%&\'()*+,:;<=>?@[\\]^`{|}~'


class Config(object):
    def __init__(self):
        self.bling = BlingConfig()
        self.commands = CommandConfig()
        self.lock = LockConfig()
        self.packer = PackerConfig()
        self.docker = DockerConfig()


config = Config()



# -*- coding: utf-8 -*-
from .git import get_current_branch
from .git import get_current_commit

from ..domain.vendor import GitError
from . import docker

# @see: https://docs.travis-ci.com/user/environment-variables/
#
# Below are the list of TravisCI environment variables we use in our builds.
travis_branch = 'TRAVIS_BRANCH'
travis_commit = 'TRAVIS_COMMIT'
REQUIRED_TRAVIS_ENV = {
    travis_branch: get_current_branch,
    travis_commit: get_current_commit,
}


def cleanse_environ(environ):
    environ = environ.copy()
    if environ.get(travis_branch):
        environ[travis_branch] = docker.cleanse_image_tag(environ[travis_branch])
    return environ


def patch_environ(environ, required_travis_env=REQUIRED_TRAVIS_ENV):
    """Updates `environ` with TRAVIS vars to simulate a build environment."""
    environ = environ.copy()
    for env in required_travis_env:
        if environ.get(env) is not None:
            continue
        try:
            environ[env] = required_travis_env[env]()
        except GitError:
            environ[env] = None
    return environ



# -*- coding: utf-8 -*-
import json

from ..domain import blingrc
from ..domain.vendor import PackerConfigNotFound
from ..vendor.packer import read_config
from ..core.io import err

from ..config import config as app_conf


class BlingConfig(object):
    def __init__(self, config):
        self._config = config
        self.custom_keys = {
            'image_name': self.image_name,
        }

        self._config = self.custom_keys.copy()
        self._config.update(config)

    @property
    def config(self):
        return self._config.copy()

    @property
    def image_name(self):
        return '{registry}/{name}'.format(**self.config)

    def get(self, key, or_else=None):
        return self.config.get(key, or_else)


def open_config():
    config_file = app_conf.bling.config_path
    try:
        with open(config_file, 'rU') as f:
            data = json.load(f)
        for key, default in app_conf.bling.optional_keys.iteritems():
            if key not in data:
                data[key] = default
        return BlingConfig(data)
    except IOError:
        raise blingrc.MissingBlingConfig('cannot find "%s" config file' % config_file)
    except ValueError as e:
        raise blingrc.MalformedBlingConfig('malformed json in "%s"\n%s' % (config_file, e))


def _validate_config(config):
    errors = []
    for key in app_conf.bling.required_keys:
        if key in config.config.keys():
            continue
        errors.append('expected key: "%s" key (not found)' % key)
    try:
        read_config(config.get('packer', ''))
    except PackerConfigNotFound as e:
        errors.append('%s (packer missing)' % e)
    return blingrc.ConfigValidation(errors=errors, config=config)


def validate_config(config):
    validation = _validate_config(config)
    if not validation.has_error:
        return True
    else:
        [err(error) for error in validation.errors]
        return False


def patch_environ(environ, config):
    environ = environ.copy()
    environ.update({k: str(v) for k, v in config.iteritems()})
    return environ
